

#include <AudioUnit/AudioUnit.h>

#import "AudioUnitTest.h"

#define kOutputBus 0
#define kInputBus 1

OSStatus sStatus;
AudioComponentInstance sAudioUnit;

void checkStatus( OSStatus inStatus)
{
    printf( "CHECK STATUS: %x\n", ( unsigned int) inStatus);
}

void startAudio()
{
    // to start the system
    OSStatus oStatus = AudioOutputUnitStart( sAudioUnit);
    checkStatus( oStatus);
}

void stopAudio()
{
    // to stop the system
    OSStatus oStatus = AudioOutputUnitStop( sAudioUnit);
    checkStatus( oStatus);
}

void uninitAudio()
{
    // when we are finished
    AudioUnitUninitialize( sAudioUnit);
}

// to record
static OSStatus recordingCallback( void *inRefCon, AudioUnitRenderActionFlags *ioActionFlags, const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber, UInt32 inNumberFrames, AudioBufferList *ioData)
{
  // TODO: Use inRefCon to access our interface object to do stuff
  // Then, use inNumberFrames to figure out how much data is available, and make
  // that much space available in buffers in an AudioBufferList.

//AudioBufferList *bufferList; // <- Fill this up with buffers (you will want to malloc it, as it's a dynamic-length list)

  // Then:
  // Obtain recorded samples

//OSStatus oStatus;
//oStatus = AudioUnitRender( [audioInterface audioUnit], ioActionFlags, inTimeStamp, inBusNumber, inNumberFrames, bufferList);
//checkStatus( oStatus);

  // Now, we have the samples we just read sitting in buffers in bufferList
//DoStuffWithTheRecordedAudio( bufferList);
  return noErr;
}

// to play
static OSStatus playbackCallback( void *inRefCon, AudioUnitRenderActionFlags *ioActionFlags, const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber, UInt32 inNumberFrames, AudioBufferList *ioData)
{
//    inRefCon is the class we want

    // Notes: ioData contains buffers (may be more than one!)
    // Fill them up as much as you can. Remember to set the size value in each buffer to match how
    // much data is in the buffer.

//  stopAudio();

    return noErr;
}



void initAudio()
{
    // Describe audio component
    AudioComponentDescription oAudioComponentDescription;
    oAudioComponentDescription.componentType = kAudioUnitType_Output;
    oAudioComponentDescription.componentSubType = kAudioUnitSubType_RemoteIO;
//  oAudioComponentDescription.componentSubType = kAudioUnitSubType_DefaultOutput;
    oAudioComponentDescription.componentFlags = 0;
    oAudioComponentDescription.componentFlagsMask = 0;
    oAudioComponentDescription.componentManufacturer = kAudioUnitManufacturer_Apple;

    // Get component
    AudioComponent oInputComponent = AudioComponentFindNext( NULL, &oAudioComponentDescription);

    // Get audio units
    sStatus = AudioComponentInstanceNew( oInputComponent, &sAudioUnit);
    checkStatus( sStatus);

    // Enable IO for recording
    UInt32 oFlag = 1;
//  sStatus = AudioUnitSetProperty( sAudioUnit, kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Input, kInputBus,&oFlag, sizeof( oFlag));
    checkStatus( sStatus);

    // Enable IO for playback
    sStatus = AudioUnitSetProperty( sAudioUnit, kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Output, kOutputBus, &oFlag, sizeof( oFlag));
    checkStatus( sStatus);

    // Describe format
    AudioStreamBasicDescription oAudioFormat;
    oAudioFormat.mSampleRate         = 44100.00;
    oAudioFormat.mFormatID           = kAudioFormatLinearPCM;
    oAudioFormat.mFormatFlags        = kAudioFormatFlagIsSignedInteger | kAudioFormatFlagIsPacked;
    oAudioFormat.mFramesPerPacket    = 1;
    oAudioFormat.mChannelsPerFrame   = 1;
    oAudioFormat.mBitsPerChannel     = 16;
    oAudioFormat.mBytesPerPacket     = 2;
    oAudioFormat.mBytesPerFrame      = 2;

    // Apply format
    sStatus = AudioUnitSetProperty( sAudioUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Output, kInputBus, &oAudioFormat, sizeof( oAudioFormat));
    checkStatus( sStatus);

//  sStatus = AudioUnitSetProperty( sAudioUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input, kOutputBus, &oAudioFormat, sizeof( oAudioFormat));
//  checkStatus( sStatus);

    // Set input callback
//  AURenderCallbackStruct callbackStruct;
//  callbackStruct.inputProc = recordingCallback;
//  callbackStruct.inputProcRefCon = self;
//  sStatus = AudioUnitSetProperty( sAudioUnit, kAudioOutputUnitProperty_SetInputCallback, kAudioUnitScope_Global, kInputBus, &callbackStruct, sizeof( callbackStruct));
//  checkStatus( sStatus);

    // Set output callback
    AURenderCallbackStruct oCallbackStruct;
    oCallbackStruct.inputProc = playbackCallback;
//  oCallbackStruct.inputProcRefCon = self;
    sStatus = AudioUnitSetProperty( sAudioUnit, kAudioUnitProperty_SetRenderCallback, kAudioUnitScope_Global, kOutputBus, &oCallbackStruct, sizeof( oCallbackStruct));
    checkStatus( sStatus);

    // Disable buffer allocation for the recorder (optional - do this if we want to pass in our own)
    oFlag = 0;
    sStatus = AudioUnitSetProperty( sAudioUnit, kAudioUnitProperty_ShouldAllocateBuffer, kAudioUnitScope_Output, kInputBus, &oFlag, sizeof( oFlag));

    // Initialise
    sStatus = AudioUnitInitialize( sAudioUnit);
    checkStatus( sStatus);
}

void checkAudioUnit( const char *inFileName)
{
}
