

#include "vorbisfile.h"
#import "OggTest.h"

#define MAX_PATH            256
#define SOUND_BUFFER_SIZE   65536

typedef struct
{
    char            mFileName[ MAX_PATH];

    unsigned int    mPos;
    unsigned int    mSize;
    
    unsigned char   *mpBuffer;
    unsigned char   *mpCurrent;
    void            *mpUserdata;

} BCStream;


BCStream *streamOpen( const char *inFileName)
{
    FILE *opFile;

    BCStream *opStream;
    opFile = fopen( inFileName, "rb");

    if ( opFile == NULL)
    { 
        return NULL; 
    }

    opStream = ( BCStream *) calloc( 1, sizeof( BCStream));
    memcpy( opStream->mFileName, inFileName, strlen( inFileName) + 1);

    fseek( opFile, 0, SEEK_END);
    opStream->mSize = ftell( opFile);
    fseek( opFile, 0, SEEK_SET);

    opStream->mpBuffer = ( unsigned char * ) malloc( opStream->mSize);
    fread( opStream->mpBuffer, 1, opStream->mSize, opFile);

    opStream->mpCurrent = opStream->mpBuffer;

    fclose( opFile);
    return opStream;
}

BCStream *streamClose( BCStream *inpStream)
{
    if ( inpStream == NULL)
    {   
        return NULL; 
    }
    
    if ( inpStream->mpBuffer)
    {
        free( inpStream->mpBuffer);
        inpStream->mpBuffer = NULL;
    }

    free( inpStream);
    return NULL;
}


size_t oggRead( void *inpOggBuffer, size_t inSize, size_t inRead, void *inpStream)
{
    unsigned int oBytesLeft;
    unsigned int oPos;
            
    BCStream *opStream  = ( BCStream *) inpStream;
    oBytesLeft = opStream->mSize - opStream->mPos;

    if (( inRead * inSize) < oBytesLeft)
    { 
        oPos = inRead * inSize; 
    }
    else
    { 
        oPos = oBytesLeft; 
    }
    
    if ( oPos != 0)
    {
        memcpy( inpOggBuffer, opStream->mpBuffer + opStream->mPos, oPos);
        opStream->mPos += oPos;
    }

    return oPos;
}

int oggSeek( void *inpStream, ogg_int64_t inOffset, int inStride)
{
    unsigned int oPos;
    unsigned int oBytesLeft;
    BCStream *opStream = ( BCStream *) inpStream;

    switch( inStride)
    {
        case SEEK_SET:
        {
            if ( opStream->mSize >= inOffset)
            { 
                oPos = inOffset; 
            }
            else
            { 
                oPos = opStream->mSize;
            }

            opStream->mPos = oPos;
            break;
        }
        
        case SEEK_CUR:
        {
            oBytesLeft = opStream->mSize - opStream->mPos;
            
            if ( inOffset < oBytesLeft)
            { 
                oPos = inOffset; 
            }
            else
            { 
                oPos = oBytesLeft; 
            }
        
            opStream->mPos += oPos;
            break;
        }

        case SEEK_END:
        {
            opStream->mPos = opStream->mSize + 1;
            break;
        }
    };

    return 0;
}

long oggTell( void *inpStream)
{
    BCStream *opStream = ( BCStream * ) inpStream;
    return opStream->mPos;
}

int oggClose( void *inpStream)
{
    if ( inpStream)
    { 
        return 1; 
    }
    else
    { 
        return 0; 
    }
}



void checkVorbis( const char *inFileName)
{
    BCStream *opStream = streamOpen( inFileName);

    OggVorbis_File oOggVorbisFile;
    vorbis_info *opVorbisInfo;

    ov_callbacks oOvCallbacks;
    oOvCallbacks.read_func  = oggRead;
    oOvCallbacks.seek_func  = oggSeek;
    oOvCallbacks.tell_func  = oggTell;
    oOvCallbacks.close_func = oggClose;

    ov_open_callbacks( opStream, &oOggVorbisFile, NULL, 0, oOvCallbacks);

    opVorbisInfo = ov_info( &oOggVorbisFile, -1);
    
    if ( opVorbisInfo->channels == 1)
    { 
//      _SIO2soundbuffer->format = AL_FORMAT_MONO16;
    }
    else
    { 
//      _SIO2soundbuffer->format = AL_FORMAT_STEREO16;
    }

    int oSize = ( ov_pcm_total( &oOggVorbisFile, -1) * opVorbisInfo->channels << 1);
    char *opData = ( char * ) malloc( oSize);
//  int oRate = opVorbisInfo->rate;
//  printf( "VORBIS INFO: %d %d\n", oSize, oRate);

    char *opWriteData = opData;
    int oCount;
    int oBit;

    while (( oCount = ov_read( &oOggVorbisFile, opWriteData, SOUND_BUFFER_SIZE, 0, 2, 1, &oBit)) > 0)
    {
        opWriteData += oCount;
    }
    
    ov_clear( &oOggVorbisFile);
    free( opData);

    streamClose( opStream);
}


